/*  my_header.h */
/* 	Copyright 2004 Oswaldo Morizaki */
/* 	This file is part of ga-nn-ag 

    ga-nn-ag is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ga-nn-ag is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ga-nn-ag; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/un.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <netdb.h>
#include <unistd.h>
#include <syslog.h>
#include <errno.h>
#include <fcntl.h>
#include <setjmp.h>

#include "stdlib.h"
#include "stdio.h"
#include "math.h"
#include "strings.h"

#define BUFFSIZE 256
#define LISTENQ 1024
#define FILE_MODE (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)

#define NUM_TYPE 2
#define MAX_MOMENTUM 6

char char_buffer[BUFFSIZE];

/*
ga_errno:

0 = no error
1 = memory allocation error
*/
int ga_errno;

struct nn_config
{
	int num_pat;
	int num_input;
	int num_output;
	
	int mode; //mode type
	
	char input_preffix[BUFFSIZE];
	char output_preffix[BUFFSIZE];
};

struct ga_server_config
{	
	int timestamp;
	int time_wait;
	int master;

	char message[BUFFSIZE];

	char ip[BUFFSIZE];
	char init_preffix[BUFFSIZE];
	char result_preffix[BUFFSIZE];
	char ** hosts;
	char ** niches;

	unsigned int remote_server_port;
	unsigned int local_server_port;
	
	int max_local;
	int first_local;
	int last_local;
	int num_hosts;
	int num_niche;

	int poblation;
	int generations;
	int max_neuron_mut;
	int max_num_layer;
	int max_num_neuron;
	int agr;
	int age;
	
	float mut_new_prob;
	float mut_neuron_prob;
	float mut_net_prob;
	float inm_prob;
	float lifespan;
	float base;
	float boost;
	float level;
	
	float prob_alpha;
	float prob_conv_rate;
	float prob_bias_corr;
	float prob_delta_type;
	float prob_momentum;
	float prob_num_con;
	float prob_bias;
};

struct neuron
{
	float conv_rate;//convergence rate
	float bias;			//bias input
	float decayment;//memory decayment value (zero if no decayment)

	int block;			//Marker for blocks, non volatile, values same as inner
									//11 = block inner : 12 = block lower : 13 = block upper : 14 = 12 and 13
	int bias_corr;	//just zero or one
	int delta_type;	//less than NUM_TYPE
	int momentum;		//momentum level
	int dimension; 	//neuron dimension space
	int num_con;		//number of conected elements
	int inner;			//0 = not inner : 1 = inner : 2 = lower border : 3 = upper border : 4 = 2 and 3
									//11 = block inner : 12 = block lower : 13 = block upper : 14 = 12 and 13
	float * range;		//range of connection, depends on dimension
	float * x_c;			//coordinate array, depends on dimension
	float * alpha;		//One value per momentum level

	struct connection ** con;
	
	double value;		//network value
	double p_value; //previus value
	double delta;		//error value
};

struct connection
{
	int age;
	float weight;
	float * con_x;						//Size depends on dimension
	float * delta_weight;  		//Size depends on momentum
};

struct neural_net
{
	struct neuron ** neuron_array;
	int num_neuron;
	int age;
	int dimension;
};

